/* layer.c
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023, 2024 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#include <assert.h>
#include <stdarg.h>
#include <stdbool.h>

#include "layer.h"

void layer_construct(Layer *this, int width, int height, struct LinearPixel default_bgcolor)
{
	this->default_bgcolor = default_bgcolor;
	this->alpha = 1;
	this->visible = true;
	Image *_tmp_1 = (Image *) malloc(sizeof(Image));
	if(_tmp_1 == NULL) {
		perror(NULL);
		exit(EXIT_FAILURE);
	}

	image_construct(_tmp_1, width, height, 255);
	this->img = _tmp_1;
	image_fill(this->img, default_bgcolor);
}

void layer_replace_image(Layer *this, Image *newimg)
{
	Image *tmp = this->img;
	if(tmp) {
		image_destruct((Image *) tmp);
		free(tmp);
		tmp = NULL;
	}

	this->img = newimg;
}

void layer_set_alpha(Layer *this, double alpha_in)
{
	this->alpha = alpha_in;
}

void layer_set_visible(Layer *this, _Bool visible_in)
{
	this->visible = visible_in;
}

void layer_destruct(Layer *this)
{
	if(this->img) {
		image_destruct(this->img);
		free(this->img);
	}

}

