/* ui.h
 * This file is part of Vara digital painting program
 * Copyright 2021, 2023, 2024 Nandakumar Edamana
 * Distributed under GNU General Public License, version 3
 */ 

#ifndef N96_DRAW_ui_h
#define N96_DRAW_ui_h

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "defaults.h"
#include "entryslider.h"
#include "entryslider.h"
#include "brushes.h"
#include "brushes.h"
#include "doc.h"
#include "doc.h"
#include "helpmenuitem.h"
#include "helpmenuitem.h"
#include "imageview.h"
#include "imageview.h"
#include "newdocdialog.h"
#include "newdocdialog.h"
#include "quickpalette.h"
#include "quickpalette.h"
#include <cairo.h>
#define PROGRAM_NAME ("Vara")
#define VERSION ("0.1.240807")
#define DEFWIDTH_DOC (640)
#define DEFHEIGHT_DOC (480)
#define DEFWIDTH_MWND (1366)
#define DEFHEIGHT_MWND (720)
#define DEFWIDTH_CAIRO (1920)
#define DEFHEIGHT_CAIRO (1080)
#define MINWIDTH_MWND (1024)
#define MINHEIGHT_MWND (400)
#define MINWIDTH_LBL_BRUSH_SIZE (24)
#define MINWIDTH_LBL_ZOOM_SIZE (48)
typedef struct CommandLineOptions {
	_Bool static_cursor;
} CommandLineOptions;

extern CommandLineOptions clopts;
extern double ZOOMSTEP;
#define MAXWIDTH (4096)
typedef struct SimpleHelpItem {
	const char * title;
	const char * message;
} SimpleHelpItem;

extern SimpleHelpItem helpitems[2];
typedef struct myapp {
	GtkLabel *lbl_zoom;
	GtkToolItem *tbi_lbl_zoom;
	GtkToolButton *btn_zoomin;
	GtkToolButton *btn_zoomout;
	GtkToolButton *btn_zoom100;
	GtkToolButton *btn_center;
	GtkToolbar *tbr_zoom;
	GtkFrame *fme_zoom;
	QuickPalette *quickpalette;
	GtkButton *btn_layer_fillbgcolor;
	GtkBox *box_layeracts;
	GtkFrame *fme_layeracts;
	GtkCheckButton *chk_bg_visible;
	GtkCheckButton *chk_fg_visible;
	GtkBox *box_layer_visibility;
	GtkFrame *fme_layer_visibility;
	GtkRadioButton *btn_bglayer;
	GtkRadioButton *btn_fglayer;
	GtkBox *box_layers;
	GtkFrame *fme_layers;
	GtkBox *box_controls1;
	GtkButton *btn_export_doc;
	GtkBox *hbx_newdoc;
	GtkFrame *fme_newdoc;
	GtkBox *box_newdoc;
	GtkGrid *grd_bottom;
	GtkCheckButton *chk_scratch;
	GtkBox *box_sandbox;
	GtkFrame *fme_sandbox;
	GtkCheckButton *chk_subpixel_when_zoomed;
	EntrySlider *sldr_stroke_smoothness;
	GtkBox *box_strokeopts;
	GtkFrame *fme_strokeopts;
	EntrySlider *sldr_brush_aaradius_circle1;
	EntrySlider *sldr_brush_softness;
	EntrySlider *sldr_brush_opacity;
	GtkCheckButton *chk_pressure;
	GtkComboBoxText *cbo_brush_engine;
	GtkBox *box_brushopts;
	GtkFrame *fme_brushopts;
	GtkBox *box_controls3;
	GtkScrollbar *hscroll;
	GtkScrollbar *vscroll;
	GtkDrawingArea *da;
	GtkGrid *grd_viewport;
	GtkButton *btn_redo;
	GtkButton *btn_undo;
	GtkBox *box_history;
	GtkFrame *fme_history;
	GtkCheckButton *chk_advanced;
	GtkFrame *fme_advanced;
	GtkCheckButton *chk_scatter;
	GtkFrame *fme_scatter;
	GtkButton *btn_prebrush_soft_round;
	GtkButton *btn_prebrush_round_pressure_size;
	GtkButton *btn_prebrush_round;
	GtkButton *btn_prebrush_charcoal;
	GtkButton *btn_prebrush_ballpoint;
	GtkBox *box_brushes;
	GtkFrame *fme_brushes;
	ImageView *iv_brushpreview;
	GtkFrame *fme_brushpreview;
	GtkSpinButton *spn_brushsiz;
	GtkFrame *fme_brushsiz;
	GtkColorButton *btn_color;
	GtkBox *box_color;
	GtkFrame *fme_color;
	GtkRadioButton *btn_floodfill;
	GtkRadioButton *btn_eyedropper;
	GtkRadioButton *btn_eraser;
	GtkRadioButton *btn_brush;
	GtkRadioButton *btn_pencil;
	GtkBox *box_tools;
	GtkFrame *fme_tools;
	GtkBox *box_controls2;
	GtkMenuItem *mni_help_about;
	GtkMenuItem *mni_help_topics;
	GtkMenu *mnu_help;
	GtkMenuItem *mni_help;
	GtkMenuItem *mni_tools_bminus;
	GtkMenuItem *mni_tools_bplus;
	GtkMenuItem *mni_tools_eyedropper;
	GtkMenuItem *mni_tools_floodfill;
	GtkMenuItem *mni_tools_eraser;
	GtkMenuItem *mni_tools_brush;
	GtkMenuItem *mni_tools_pencil;
	GtkMenu *mnu_tools;
	GtkMenuItem *mni_tools;
	GtkMenuItem *mni_view_zoom100;
	GtkMenuItem *mni_view_zoomout;
	GtkMenuItem *mni_view_zoomin;
	GtkMenu *mnu_view;
	GtkMenuItem *mni_view;
	GtkMenuItem *mni_edit_clear;
	GtkMenuItem *mni_edit_fill_bgcolor;
	GtkMenuItem *mni_edit_fill_fgcolor;
	GtkMenuItem *mni_edit_redo;
	GtkMenuItem *mni_edit_undo;
	GtkMenu *mnu_edit;
	GtkMenuItem *mni_edit;
	GtkMenuItem *mni_file_quit;
	GtkMenuItem *mni_file_export_layer;
	GtkMenuItem *mni_file_export;
	GtkMenuItem *mni_file_new;
	GtkMenu *mnu_file;
	GtkMenuItem *mni_file;
	GtkMenuBar *mbr;
	GtkBox *box_main;
	GtkAccelGroup *accgrp;
	GtkWindow *wnd;
	_Bool was_autoscroll;
	_Bool disable_brush_preview;
	cairo_surface_t *surf;
	ImageDocument *doc;
	int curcursorsiz;
	_Bool curcursor_is_circle;
	int test_vara_logo_size;
	_Bool test_vara_logo_exit;
	ImageView *ivw_ballpoint;
	ImageView *ivw_charcoal;
	ImageView *ivw_round;
	ImageView *ivw_round_pressure_size;
	ImageView *ivw_soft_round;
} myapp;

void myapp_construct(myapp *this, ImageDocument *newdoc);
void myapp_destruct(myapp *this);
_Bool myapp_can_zoom_in(myapp *this);
_Bool myapp_can_zoom_out(myapp *this);
_Bool myapp_prompt_close_document(myapp *this);
void myapp_reflect_brush_status(myapp *this);
void myapp_reflect_document_state(myapp *this);
void myapp_reflect_color_status(myapp *this);
void myapp_reflect_tool_status(myapp *this);
void myapp_set_canvas_cursor(myapp *this, const char * name);
void myapp_set_canvas_cursor_for_active_tool(myapp *this);
void myapp_set_document(myapp *this, ImageDocument *newdoc);
void myapp_on_btn_color_set(GtkColorButton *widget, void *user_data);
void myapp_on_quickpalette_colorset(QuickPalette *widget, struct LinearPixel eventcolor_linear, void *user_data);
void myapp_on_spn_brushsiz_changed(GtkSpinButton *widget, void *user_data);
void myapp_on_btn_tool_x_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_cbo_brush_engine_changed(GtkComboBox *widget, void *user_data);
void myapp_on_mni_file_quit_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_mni_fill_bgcolor_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_mni_fill_fgcolor_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_mni_tool_x_toggled(GtkMenuItem *widget, void *user_data);
void myapp_on_mni_tools_bplus_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_mni_tools_bminus_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_btn_prebrush_clicked(GtkButton *widget, void *user_data);
void myapp_on_sldr_stroke_smoothness_changed(EntrySlider *widget, void *user_data);
void myapp_on_chk_advanced_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_chk_scatter_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_chk_scratch_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_chk_subpixel_when_zoomed_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_chk_pressure_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_btn_bglayer_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_btn_clear_layer_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_fglayer_toggled(GtkCheckButton *widget, void *user_data);
void myapp_handle_fileio_err(myapp *this, enum VaraFileIOError err);
void myapp_on_btn_export_doc_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_export_layer_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_undo_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_redo_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_center_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_zoom100_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_zoomin_clicked(GtkButton *widget, void *user_data);
void myapp_on_btn_zoomout_clicked(GtkButton *widget, void *user_data);
void myapp_on_chk_bg_visible_toggled(GtkCheckButton *widget, void *user_data);
void myapp_on_chk_fg_visible_toggled(GtkCheckButton *widget, void *user_data);
char * myapp_prompt_for_export_filename(myapp *this);
void myapp_on_da_sizeallocate(GtkDrawingArea *widget, GdkRectangle *nggevent_allocation, void *user_data);
void myapp_on_hscroll_valuechanged(GtkRange *widget, void *user_data);
void myapp_on_sldr_brush_aaradius_circle1_changed(EntrySlider *widget, void *user_data);
void myapp_on_sldr_brush_opacity_changed(EntrySlider *widget, void *user_data);
void myapp_on_sldr_brush_softness_changed(EntrySlider *widget, void *user_data);
void myapp_on_vscroll_valuechanged(GtkRange *widget, void *user_data);
void myapp_pick_tool(myapp *this, enum Tool tool, _Bool erase);
void myapp_reserve_cairo_surface(myapp *this, int w, int h);
void myapp_update_brush_size_view(myapp *this);
void myapp_update_history_controls(myapp *this);
void myapp_update_scroll_controls(myapp *this);
void myapp_update_zoom_controls(myapp *this);
void myapp_process_da_event(myapp *this, int event_type, int x, int y, double pressure, GdkScrollDirection scroll_direction);
struct VaraRect myapp_copy_preview_to_cairo_surface(myapp *this);
void myapp_clear_cairo_surface(myapp *this);
void myapp_drawframe(myapp *this);
void myapp_drawframe_noskip(myapp *this);
void myapp_on_mni_help_about_activate(GtkMenuItem *widget, void *user_data);
void myapp_on_btn_new_doc_clicked(GtkButton *widget, void *user_data);
void myapp_repreview_brush(myapp *this);
void on_da_event(GtkWidget *w, GdkEvent *event, myapp *app);
void on_da_paint(GtkWidget *w, cairo_t *cr, myapp *app);
_Bool on_wnd_deleteevent(GtkWidget *wgt, GdkEvent *event, myapp *app);
void draw_brush_preview(Image *img, struct DrawingParams dpcopy, _Bool dot_only);
ImageView * brushpreview_widget_new(struct BrushParams brushparams);
_Bool probably_inside_container();
char * append_png_if_needed_and_possible(const char * fnam);
CommandLineOptions command_line_options_default();

#endif /* N96_DRAW_ui_h */
